@@ -7,8 +7,10 @@ use File::Basename;
 use File::Copy qw(copy);
 use List::Util qw[min max];
 
+my $srcdir = dirname(__FILE__);
+my $rundir = getcwd();
+$srcdir = $rundir unless ($srcdir ne '.');
 #change directory to script location
-chdir(dirname(__FILE__));
 use lib dirname(__FILE__);
 
 use gmxFile::Path qw(remove_tree);
@@ -28,6 +30,7 @@ my $mpi_ranks = 0;
 my $double   = 0;
 my $crosscompiling = 0;
 my $bluegene = 0;
+my $keep_files = 0;
 my $verbose  = 5;
 my $xml      = 0;
 # energy file comparision tolerance (potentials, not virials or pressure)
@@ -309,7 +312,7 @@ sub check_force($$)
         }
     }     
     close(FIN);
-    if ($nerr_force == 0) {
+    if ($nerr_force == 0 && !$keep_files) {
       unlink($cfor,$cfor2);
     }
     return $nerr_force;
@@ -342,7 +345,7 @@ sub check_virial($)
 	}
     }     
     close(VIN);
-    if ($nerr_vir == 0) {
+    if ($nerr_vir == 0 && !$keep_files) {
       unlink($cvir,$cvir2);
     }
     return $nerr_vir;
@@ -470,12 +473,12 @@ sub how_should_we_rerun_mdrun {
             last;
         }
         elsif ($line =~ "Your choice of .* MPI rank.* and the use of .* total threads .* leads to the use of .* OpenMP threads, whereas we expect the optimum to be with more MPI ranks" ||
-               $line =~ "Your choice of number of MPI ranks and amount of resources results in using" ) {
-            # On large nodes this error needs to be handled.  It can
-            # be converted to a warning with setting the environment
-            # variable GMX_BYPASS_EFFICIENCY_CHECK, but we don't want
-            # anybody testing GROMACS to have to do that, whether with
-            # "make check" or stand-alone.
+               $line =~ "Your choice of number of MPI ranks and amount of resources results in using" ||
+               $line =~ "which is larger than GMX_OPENMP_MAX_THREADS") {
+            # On nodes with large core counts, the default of using
+            # all cores can lead to these errors. They need to be
+            # worked around so that anybody testing GROMACS gets a
+            # successful result unless it's actually broken.
             my $new_omp_threads = 6; # matches nthreads_omp_mpi_target_max in the code
             if ($$omp_threads_ref > 6)
             {
@@ -552,23 +555,27 @@ sub how_should_we_rerun_mdrun {
 sub run_mdrun {
     # Copy all parameters by value, which is useful so we can modify
     # them if we need to, and have the changes local to this test
-    my ($tmpi_ranks, $omp_threads, $mpi_ranks, $npme_ranks, $pme_option, $update_option, $gpu_id, $mdprefix, $mdparams, $grompp_mdp) = @_;
+    my ($tmpi_ranks, $omp_threads, $mpi_ranks, $npme_ranks, $pme_option, $update_option, $gpu_id, $mdprefix, $mdparams, $grompp_mdp, $input_dir) = @_;
     # Only one of tmpi_ranks or mpi_ranks may be greater than zero, but
     # this is checked for sanity after parsing user input.
 
     # Set up and enforce the maximum number of OpenMP threads to
     # try for this test case
-    if ( -f $max_openmp_threads_filename )
-    {
-        open my $fh, '<', $max_openmp_threads_filename or die "error opening $max_openmp_threads_filename: $!";
-        $omp_threads = do { local $/; <$fh> };
-        chomp $omp_threads;
+    my $max_omp_threads = undef;
+    if ( -f "$input_dir/$max_openmp_threads_filename" ) {
+        open my $fh, '<', "$input_dir/$max_openmp_threads_filename" or die "error opening $input_dir/$max_openmp_threads_filename: $!";
+        $max_omp_threads = do { local $/; <$fh> };
+        chomp $max_omp_threads;
     }
+    else {
+        $max_omp_threads = 64;
+    }
+    $omp_threads = ($omp_threads < $max_omp_threads) ? $omp_threads : $max_omp_threads;
 
     # Set up and enforce the maximum number of MPI ranks to try
     # for this test case
-    if ( -f $max_mpi_ranks_filename ) {
-        open my $fh, '<', $max_mpi_ranks_filename or die "error opening $max_mpi_ranks_filename: $!";
+    if ( -f "$input_dir/$max_mpi_ranks_filename" ) {
+        open my $fh, '<', "$input_dir/$max_mpi_ranks_filename" or die "error opening $input_dir/$max_mpi_ranks_filename: $!";
         my $max_ranks = do { local $/; <$fh> };
         chomp $max_ranks;
         if ($mpi_ranks > 0) {
@@ -635,7 +642,7 @@ sub run_mdrun {
         # sense, but removing that annotation file and instead adding
         # "supports-nb-on-gpu" to all the other test cases is
         # something that we should fix later.
-        my $test_supports_nb_on_gpu = ! -f "no-nb-gpu-support";
+        my $test_supports_nb_on_gpu = ! -f "$input_dir/no-nb-gpu-support";
         my $nb_task_assignment_opt;
         if($test_supports_nb_on_gpu)
         {
@@ -675,6 +682,7 @@ sub test_case {
     my $success = 0;
 
     my $cwd = getcwd();
+    mkdir($dir) unless (-d $dir);
     chdir($dir);
     if ($verbose > 1) {
         print "Testing $test_name . . . ";
@@ -688,7 +696,7 @@ sub test_case {
     my $grompp_mdp = "$input_dir/grompp.mdp";
     my $grompp_out = "grompp.out";
     my $grompp_err = "grompp.err";
-    $nerror = do_system("$progs{'grompp'} -f $grompp_mdp -c $input_dir/conf -r $input_dir/conf -p $input_dir/topol -maxwarn 10 $ndx >$grompp_out 2>$grompp_err");
+    $nerror = do_system("$progs{'grompp'} -f $grompp_mdp -c $input_dir/conf -r $input_dir/conf -p $input_dir/topol -ref $input_dir/rotref -maxwarn 10 $ndx >$grompp_out 2>$grompp_err");
 
     my @error_detail;
     if (! -f "topol.tpr") {
@@ -715,7 +723,7 @@ sub test_case {
                 print "\nThe GROMACS version being tested may be older than the reference version.\nPlease see the note at end of this output.\n";
                 $addversionnote = 1;
             }
-            if ($nerror == 0) {
+            if ($nerror == 0 && !$keep_files) {
                 unlink($tprout,$tprerr);
             }
         }
@@ -759,6 +767,11 @@ sub test_case {
                 $line1 =~ s/file .*grompp.mdp/file grompp.mdp/;
                 $line2 =~ s/file .*grompp.mdp/file grompp.mdp/;
 
+                # Hack to avoid useless warnings about statements being on the
+                # wrong line but otherwise being the same
+                $line1 =~ s/grompp.mdp, line.*/grompp.mdp, line/;
+                $line2 =~ s/grompp.mdp, line.*/grompp.mdp, line/;
+
                 if ("$line2" ne "$line1") {
                     $nerror++;
                     # Note that the next line uses the fact that
@@ -774,7 +787,7 @@ sub test_case {
             if ($nerror>0) {
                 print("Different warnings in $ref_warn and $grompp_warn\n");
                 push(@error_detail, ($grompp_err, $grompp_out));
-            } else {
+            } elsif (!$keep_files) {
                 unlink($grompp_warn);
             }
         }
@@ -805,7 +818,7 @@ sub test_case {
         if ($test_name =~ /-update-cpu/) {
             $update_option = "-update cpu";
         }
-        $nerror = run_mdrun($tmpi_ranks, $omp_threads, $mpi_ranks, $npme_ranks, $pme_option, $update_option, $local_gpu_id, $mdprefix, $local_mdparams, $grompp_mdp);
+        $nerror = run_mdrun($tmpi_ranks, $omp_threads, $mpi_ranks, $npme_ranks, $pme_option, $update_option, $local_gpu_id, $mdprefix, $local_mdparams, $grompp_mdp, $input_dir);
         if ($nerror != 0) {
             if ($parse_cmd eq '') {
                 push(@error_detail, ("mdrun.out", "md.log"));
@@ -847,7 +860,7 @@ sub test_case {
                 push(@error_detail, "checkvir.out ($nerr_vir errors)") if ($nerr_vir > 0);
 
                 $nerror |= $nerr_pot | $nerr_vir;
-                unlink($potout,$poterr) unless $nerr_pot;
+                unlink($potout,$poterr) unless ($nerr_pot || $keep_files);
             }
             my $reftrr = "$input_dir/${ref}.trr";
             if (! -f $reftrr ) {
@@ -943,7 +956,9 @@ sub test_case {
                 }
                 else {
                     print "PASSED\n";
-                    unlink("mdout.mdp");
+                    if (! $keep_files) {
+                        unlink("mdout.mdp");
+                    }
                 }
             }
         }
@@ -955,12 +970,12 @@ sub test_case {
 }
 
 sub prepare_run_with_different_task_decomposition {
-    my ($test_name, $suffix, $new_dir_ref, $new_input_dir_ref, $new_test_name_ref) = @_;
+    my ($test_name, $suffix, $input_dir, $new_dir_ref, $new_input_dir_ref, $new_test_name_ref) = @_;
 
     # Set up new variables that control where the test
     # case does its I/O
     $$new_dir_ref = "${test_name}/$suffix";
-    $$new_input_dir_ref = "..";
+    $$new_input_dir_ref = "$input_dir";
     $$new_test_name_ref = "${test_name}-${suffix}";
     mkdir $$new_dir_ref;
 
@@ -973,12 +988,12 @@ sub prepare_run_with_different_task_decomposition {
 }
 
 sub test_systems {
-    my ($npassed, $nn, @subdirs) = @_;
+    my ($npassed, $nn, $dirs, @subdirs) = @_;
     $$npassed = 0;
     $$nn = 0;
     foreach my $dir ( @subdirs ) {
         my $test_name = $dir;
-        my $input_dir = ".";
+        my $input_dir = "$srcdir/$dirs/$dir";
         $$nn++;
         # Run the test case
         $$npassed += test_case $dir, $input_dir, $test_name;
@@ -987,7 +1002,7 @@ sub test_systems {
         # kernels. If GPU kernels were used to run this test case
         # (whether natively or in emulation), run it again in CPU-only
         # mode. This relies on test_case() not clearing up md.log.
-        my $mdrun_ran_nb_on_gpu = ($test_name =~ /nbnxn/ && 0 < find_in_file("^Using .* 8x8 non-bonded kernels", "$dir/md.log"));
+        my $mdrun_ran_nb_on_gpu = (find_in_file("^Using .* 8x8 nonbonded short-range kernels", "$dir/md.log") > 0);
         my $specified_nb_option = ($mdparams =~ /-nb/);
         if ($mdrun_ran_nb_on_gpu) {
             if ($specified_nb_option) {
@@ -999,7 +1014,7 @@ sub test_systems {
                 print("Re-running $test_name using only CPU-based non-bonded kernels\n");
                 # Note that this also forces PME to run on the CPU, where applicable.
                 my ($new_dir, $new_input_dir, $new_test_name);
-                prepare_run_with_different_task_decomposition($test_name, "nb-cpu", \$new_dir, \$new_input_dir, \$new_test_name);
+                prepare_run_with_different_task_decomposition($test_name, "nb-cpu", $input_dir, \$new_dir, \$new_input_dir, \$new_test_name);
                 $$nn++;
                 $$npassed += test_case $new_dir, $new_input_dir, $new_test_name;
             }
@@ -1021,7 +1036,7 @@ sub test_systems {
                 print("Re-running $test_name using CPU-based PME\n");
 
                 my ($new_dir, $new_input_dir, $new_test_name);
-                prepare_run_with_different_task_decomposition($test_name, "pme-cpu", \$new_dir, \$new_input_dir, \$new_test_name);
+                prepare_run_with_different_task_decomposition($test_name, "pme-cpu", $input_dir, \$new_dir, \$new_input_dir, \$new_test_name);
                 $$nn++;
                 $$npassed += test_case $new_dir, $new_input_dir, $new_test_name;
             }
@@ -1041,7 +1056,7 @@ sub test_systems {
                 print("Re-running $test_name using CPU-based update\n");
 
                 my ($new_dir, $new_input_dir, $new_test_name);
-                prepare_run_with_different_task_decomposition($test_name, "update-cpu", \$new_dir, \$new_input_dir, \$new_test_name);
+                prepare_run_with_different_task_decomposition($test_name, "update-cpu", $input_dir, \$new_dir, \$new_input_dir, \$new_test_name);
                 $$nn++;
                 $$npassed += test_case $new_dir, $new_input_dir, $new_test_name;
             }
@@ -1085,12 +1100,15 @@ sub refcleandir {
 
 sub test_dirs {
     my $dirs = shift;
-    chdir($dirs);
+    chdir("$srcdir/$dirs");
     # glob all files, but retain only directories that match the regular expression
     my @subdirs = map { (-d $_ && $_ =~ $only_subdir) ? $_ : () } <*>;
+    chdir($rundir);
+    mkdir($dirs) unless (-d $dirs);
+    chdir($dirs);
     print XML "<testsuite name=\"$dirs\">\n" if ($xml);
     my ($nn, $npassed);
-    test_systems(\$npassed, \$nn, @subdirs);
+    test_systems(\$npassed, \$nn, $dirs, @subdirs);
     print XML "</testsuite>\n" if ($xml);
     my $nerror=0;
     if ($npassed < $nn) {
@@ -1449,30 +1467,35 @@ sub run_single_ed_system {
   my $makeEdi_err = "make_edi.err";
   my $pme_option = "";
   my $update_option = "";
+  my $input_dir = "$srcdir/essentialdynamics/$dir";
 
   if ($verbose > 1) {
       print "Testing $dir . . .\n";
   }
 
   # Make the .tpr file
-  $nerror += do_system("$progs{'grompp'} -maxwarn 1 >$grompp_out 2>$grompp_err");
+  $nerror += do_system("$progs{'grompp'} -f $input_dir/grompp.mdp -c $input_dir/conf -r $input_dir/conf -p $input_dir/topol -maxwarn 1 >$grompp_out 2>$grompp_err");
 
   if ($ediArgs) {
       # Make the essential dynamics .edi input file
       unlink $edifn;  # delete old .edi file (if any)
-      $nerror += do_system("echo $inArgs | $progs{'make_edi'} -f eigenvec.trr $ediArgs -o $edifn 1>$makeEdi_out 2>$makeEdi_err");
+      $nerror += do_system("echo $inArgs | $progs{'make_edi'} -f $input_dir/eigenvec.trr $ediArgs -o $edifn 1>$makeEdi_out 2>$makeEdi_err");
       # Check whether we get the expected .edi file
-      my $nerr = compare_textfiles($edifn, "sam_reference.edi");
+      my $nerr = compare_textfiles($edifn, "$input_dir/sam_reference.edi");
       if ( $nerr > 0 ) {
           $nerror += $nerr;
           printf("Essential dynamics '$dir' FAILED: Produced .edi file does not match the reference!\n");
       }
-  } 
+  } else {
+      $edifn = "$input_dir/$edifn";
+  }
 
   # Make a short simulation with essential dynamics constraints:
   if ( $nerror == 0 ) { # If we already had errors, there is no use in going on ...
-    unlink $edofn;  # delete old essential dynamics .xvg output file (if any)
-    $nerror = run_mdrun($tmpi_ranks, $omp_threads, $mpi_ranks, $npme_ranks, $pme_option, $update_option, $gpu_id, $mdprefix, "-ei $edifn -eo $edofn", "grompp.mdp");
+    if (!$keep_files) {
+      unlink $edofn;  # delete old essential dynamics .xvg output file (if any)
+    }
+    $nerror = run_mdrun($tmpi_ranks, $omp_threads, $mpi_ranks, $npme_ranks, $pme_option, $update_option, $gpu_id, $mdprefix, "-ei $edifn -eo $edofn", "grompp.mdp", $input_dir);
   }
 
   return $nerror;
@@ -1484,24 +1507,28 @@ sub test_essentialdynamics {
   my $retval = 0;
   my $ntest  = 0;
   my $edref  = "edsam_reference.xvg";
+  my $prefix = "$srcdir/essentialdynamics";
   my $dir = "";
 
-  chdir("essentialdynamics");
+  $dir = "essentialdynamics";
+  mkdir($dir) unless (-d $dir);
+  chdir $dir;
 
   # ----------------------------------------------------------------------------
   # Test fixed-step linear expansion along the first eigenvector. In this test,
   # the projection on EV 1 should increase by 0.0013 nm at every step.
   $dir = "linfix";
   $ntest++;
+  mkdir($dir) unless (-d $dir);
   chdir $dir || die;
   $retval = run_single_ed_system($dir, "-outfrq 1 -linfix 1 -linstep 0.0013", "0");
   if ( 0 == $retval ) {
       # Compare the essential dynamics output to the reference. This
       # makes sense only if the system was successfully run, otherwise we will
       # get a huge bunch of errors.
-      $nerror += compare_xvg_column($edref, "$dir.xvg", 0, 0.0   );  # 0 = Time (ps), must match exacly!
-      $nerror += compare_xvg_column($edref, "$dir.xvg", 1, 0.05  );  # 1 = RMSD (nm), just making sure it does not diverge completely
-      $nerror += compare_xvg_column($edref, "$dir.xvg", 2, 1.0e-6);  # 2 = EV1 projection (nm) LINFIX, last value is the tolerance
+      $nerror += compare_xvg_column("$prefix/$dir/$edref", "$dir.xvg", 0, 0.0   );  # 0 = Time (ps), must match exacly!
+      $nerror += compare_xvg_column("$prefix/$dir/$edref", "$dir.xvg", 1, 0.05  );  # 1 = RMSD (nm), just making sure it does not diverge completely
+      $nerror += compare_xvg_column("$prefix/$dir/$edref", "$dir.xvg", 2, 1.0e-6);  # 2 = EV1 projection (nm) LINFIX, last value is the tolerance
   } else {
   	  $nerror++;
   }
@@ -1513,13 +1540,14 @@ sub test_essentialdynamics {
   # others rejected.
   $dir = "linacc";
   $ntest++;
+  mkdir($dir) unless (-d $dir);
   chdir $dir || die;
   $retval = run_single_ed_system($dir, "-outfrq 2 -linacc 1 -accdir +1", "0");
   if ( 0 == $retval ) {
       # Compare the essential dynamics output to the reference:
-      $nerror += compare_xvg_column($edref, "$dir.xvg", 0, 0.0   );  # 0 = Time (ps), must match exacly!
-      $nerror += compare_xvg_column($edref, "$dir.xvg", 1, 0.005 );  # 1 = RMSD (nm)
-      $nerror += compare_xvg_column($edref, "$dir.xvg", 2, 0.001 );  # 2 = EV projection (nm) LINACC, last value is the tolerance
+      $nerror += compare_xvg_column("$prefix/$dir/$edref", "$dir.xvg", 0, 0.0   );  # 0 = Time (ps), must match exacly!
+      $nerror += compare_xvg_column("$prefix/$dir/$edref", "$dir.xvg", 1, 0.005 );  # 1 = RMSD (nm)
+      $nerror += compare_xvg_column("$prefix/$dir/$edref", "$dir.xvg", 2, 0.001 );  # 2 = EV projection (nm) LINACC, last value is the tolerance
       $nerror += check_monotonicity("$dir.xvg",     2, +1, 5e-7  );  # EV 1 projection must increase monotonically
   } else {
       $nerror++;
@@ -1531,13 +1559,14 @@ sub test_essentialdynamics {
   # should increase by 0.002 nm per step in this test.
   $dir = "radfix";
   $ntest++;
+  mkdir($dir) unless (-d $dir);
   chdir $dir || die;
   $retval = run_single_ed_system($dir, "-outfrq 1 -radfix 1-2 -radstep 0.002", "0");
   if ( 0 == $retval ) {
       # Compare the essential dynamics output to the reference:
-      $nerror += compare_xvg_column($edref, "$dir.xvg", 0, 0.0   );  # 0 = Time (ps), must match exacly!
-      $nerror += compare_xvg_column($edref, "$dir.xvg", 1, 0.05  );  # 1 = RMSD (nm)
-      $nerror += compare_xvg_column($edref, "$dir.xvg", 4, 0.0   );  # 4 = RADFIX radius (nm), must match exactly
+      $nerror += compare_xvg_column("$prefix/$dir/$edref", "$dir.xvg", 0, 0.0   );  # 0 = Time (ps), must match exacly!
+      $nerror += compare_xvg_column("$prefix/$dir/$edref", "$dir.xvg", 1, 0.05  );  # 1 = RMSD (nm)
+      $nerror += compare_xvg_column("$prefix/$dir/$edref", "$dir.xvg", 4, 0.0   );  # 4 = RADFIX radius (nm), must match exactly
       $nerror += check_radius("$dir.xvg", 2, 3, 4, 1e-5, 0);         # RADFIX radius is calculated from columns 2-3, must match column 4
   } else {
       $nerror++;
@@ -1550,12 +1579,13 @@ sub test_essentialdynamics {
   # radius should increase monotonically.
   $dir = "radacc";
   $ntest++;
+  mkdir($dir) unless (-d $dir);
   chdir $dir || die;
   $retval = run_single_ed_system($dir, "-outfrq 3 -radacc 1-2", "0");
   if ( 0 == $retval ) {
       # Compare the essential dynamics output to the reference:
-      $nerror += compare_xvg_column($edref, "$dir.xvg", 0, 0.0    );  # 0 = Time (ps), must match exacly!
-      $nerror += compare_xvg_column($edref, "$dir.xvg", 1, 0.05   );  # 1 = RMSD (nm)
+      $nerror += compare_xvg_column("$prefix/$dir/$edref", "$dir.xvg", 0, 0.0    );  # 0 = Time (ps), must match exacly!
+      $nerror += compare_xvg_column("$prefix/$dir/$edref", "$dir.xvg", 1, 0.05   );  # 1 = RMSD (nm)
       $nerror += check_radius("$dir.xvg", 2, 3, 4, 1e-5, 0);          # RADACC radius is calculated from columns 2-3, must match column 4
       $nerror += check_monotonicity("$dir.xvg", 4, +1, 0.0);          # RADACC radius must increase monotonically
   } else {
@@ -1567,12 +1597,13 @@ sub test_essentialdynamics {
   # Test acceptance radius contraction along eigenvectors 1-2 towards target structure.
   $dir = "radcon";
   $ntest++;
+  mkdir($dir) unless (-d $dir);
   chdir $dir || die;
-  $retval = run_single_ed_system($dir, "-outfrq 1 -radcon 1-2 -tar target.pdb", "0 0");
+  $retval = run_single_ed_system($dir, "-outfrq 1 -radcon 1-2 -tar $prefix/$dir/target.pdb", "0 0");
   if ( 0 == $retval ) {
       # Compare the essential dynamics output to the reference:
-      $nerror += compare_xvg_column($edref, "$dir.xvg", 0, 0.0    );  # 0 = Time (ps), must match exacly!
-      $nerror += compare_xvg_column($edref, "$dir.xvg", 1, 0.05   );  # 1 = RMSD (nm)
+      $nerror += compare_xvg_column("$prefix/$dir/$edref", "$dir.xvg", 0, 0.0    );  # 0 = Time (ps), must match exacly!
+      $nerror += compare_xvg_column("$prefix/$dir/$edref", "$dir.xvg", 1, 0.05   );  # 1 = RMSD (nm)
       $nerror += check_radius("$dir.xvg", 2, 3, 4, 1e-5, 1);          # RADCON radius is calculated from columns 2-3, must match column 4
       $nerror += check_monotonicity("$dir.xvg", 4, -1, 0.0);          # RADCON radius must decrease monotonically towards zero.
   } else {
@@ -1586,18 +1617,19 @@ sub test_essentialdynamics {
   # using a small tolerance.
   $dir = "flooding1";
   $ntest++;
+  mkdir($dir) unless (-d $dir);
   chdir $dir || die;
   $retval = run_single_ed_system($dir, "", "");
   if ( 0 == $retval ) {
       # Compare the essential dynamics output to the reference:
-      $nerror += compare_xvg_column($edref, "$dir.xvg", 0, 0.0    );  # 0 = Time (ps), must match exacly!
-      $nerror += compare_xvg_column($edref, "$dir.xvg", 1, 1e-5   );  # 1 = RMSD (nm)
-      $nerror += compare_xvg_column($edref, "$dir.xvg", 2, 1e-5   );  # 2 = EV 1 projection (nm)
-      $nerror += compare_xvg_column($edref, "$dir.xvg", 3, 1      );  # 3 = EV 1 V_fl (kJ/mol) 
-      $nerror += compare_xvg_column($edref, "$dir.xvg", 4, 1      );  # 4 = EV 1 fl_forces (kJ/mol/nm) 
-      $nerror += compare_xvg_column($edref, "$dir.xvg", 5, 1e-5   );  # 5 = EV 2 projection (nm)
-      $nerror += compare_xvg_column($edref, "$dir.xvg", 6, 1      );  # 6 = EV 2 V_fl (kJ/mol) 
-      $nerror += compare_xvg_column($edref, "$dir.xvg", 7, 1      );  # 7 = EV 2 fl_forces (kJ/mol/nm) 
+      $nerror += compare_xvg_column("$prefix/$dir/$edref", "$dir.xvg", 0, 0.0    );  # 0 = Time (ps), must match exacly!
+      $nerror += compare_xvg_column("$prefix/$dir/$edref", "$dir.xvg", 1, 1e-5   );  # 1 = RMSD (nm)
+      $nerror += compare_xvg_column("$prefix/$dir/$edref", "$dir.xvg", 2, 1e-5   );  # 2 = EV 1 projection (nm)
+      $nerror += compare_xvg_column("$prefix/$dir/$edref", "$dir.xvg", 3, 1      );  # 3 = EV 1 V_fl (kJ/mol)
+      $nerror += compare_xvg_column("$prefix/$dir/$edref", "$dir.xvg", 4, 1      );  # 4 = EV 1 fl_forces (kJ/mol/nm)
+      $nerror += compare_xvg_column("$prefix/$dir/$edref", "$dir.xvg", 5, 1e-5   );  # 5 = EV 2 projection (nm)
+      $nerror += compare_xvg_column("$prefix/$dir/$edref", "$dir.xvg", 6, 1      );  # 6 = EV 2 V_fl (kJ/mol)
+      $nerror += compare_xvg_column("$prefix/$dir/$edref", "$dir.xvg", 7, 1      );  # 7 = EV 2 fl_forces (kJ/mol/nm)
   } else {
       $nerror++;
   }
@@ -1608,18 +1640,19 @@ sub test_essentialdynamics {
   # over 50 time steps, therefore using larger tolerances.
   $dir = "flooding2";
   $ntest++;
+  mkdir($dir) unless (-d $dir);
   chdir $dir || die;
   $retval = run_single_ed_system($dir, "", "");
   if ( 0 == $retval ) {
       # Compare the essential dynamics output to the reference:
-      $nerror += compare_xvg_column($edref, "$dir.xvg", 0, 0.0    );  # 0 = Time (ps), must match exacly!
-      $nerror += compare_xvg_column($edref, "$dir.xvg", 1, 0.05   );  # 1 = RMSD (nm)
-      $nerror += compare_xvg_column($edref, "$dir.xvg", 2, 0.025  );  # 2 = EV 1 projection (nm)
-      $nerror += compare_xvg_column($edref, "$dir.xvg", 3, 250    );  # 3 = EV 1 V_fl (kJ/mol) 
-      $nerror += compare_xvg_column($edref, "$dir.xvg", 4, 1000   );  # 4 = EV 1 fl_forces (kJ/mol/nm) 
-      $nerror += compare_xvg_column($edref, "$dir.xvg", 5, 0.025  );  # 5 = EV 2 projection (nm)
-      $nerror += compare_xvg_column($edref, "$dir.xvg", 6, 250    );  # 6 = EV 2 V_fl (kJ/mol) 
-      $nerror += compare_xvg_column($edref, "$dir.xvg", 7, 1000   );  # 7 = EV 2 fl_forces (kJ/mol/nm) 
+      $nerror += compare_xvg_column("$prefix/$dir/$edref", "$dir.xvg", 0, 0.0    );  # 0 = Time (ps), must match exacly!
+      $nerror += compare_xvg_column("$prefix/$dir/$edref", "$dir.xvg", 1, 0.05   );  # 1 = RMSD (nm)
+      $nerror += compare_xvg_column("$prefix/$dir/$edref", "$dir.xvg", 2, 0.025  );  # 2 = EV 1 projection (nm)
+      $nerror += compare_xvg_column("$prefix/$dir/$edref", "$dir.xvg", 3, 250    );  # 3 = EV 1 V_fl (kJ/mol)
+      $nerror += compare_xvg_column("$prefix/$dir/$edref", "$dir.xvg", 4, 1000   );  # 4 = EV 1 fl_forces (kJ/mol/nm)
+      $nerror += compare_xvg_column("$prefix/$dir/$edref", "$dir.xvg", 5, 0.025  );  # 5 = EV 2 projection (nm)
+      $nerror += compare_xvg_column("$prefix/$dir/$edref", "$dir.xvg", 6, 250    );  # 6 = EV 2 V_fl (kJ/mol)
+      $nerror += compare_xvg_column("$prefix/$dir/$edref", "$dir.xvg", 7, 1000   );  # 7 = EV 2 fl_forces (kJ/mol/nm)
   } else {
       $nerror++;
   }
@@ -1651,6 +1684,7 @@ Usage: ./gmxtest.pl [ -np N ] [ -nt 1 ] [ -npme n ]
                     [ -suffix xxx ] [ -reprod ] [ -mpirun mpirun_command ]
                     [ -mdrun mdrun_command ]
                     [ -crosscompile ] [ -relaxed ] [ -tight ] [ -mdparam xxx ]
+                    [ -keep ]
                     [ $dirs | all ]
 or:    ./gmxtest.pl clean | refclean | dist
 EOP
@@ -1737,6 +1771,9 @@ for ($kk=0; ($kk <= $#ARGV); $kk++) {
 	$bluegene = 1;
 	print "Will test BlueGene. You should probably set '-mpirun runjob' if you have not already.\n";
     }
+    elsif ($arg eq '-keep') {
+        $keep_files = 1;
+    }
     elsif ($arg eq '-np' ) {
 	if ($kk <$#ARGV) {
 	    $kk++;
